import axios from 'axios';
import { config } from '../../config';
const base = "http://localhost:" + config.internal_port

let access_token;
let axios_config;

beforeAll(async () => {
    const res = await axios.post(base + '/api/auth/login', { username: "demo", password: "demo" });
    access_token = res.data["access_token"];
    axios_config = {
        headers: { "authorization": "Bearer " + access_token },
        validateStatus: undefined
    };
});

describe('Update contact name after adding', () => {
    let added_contact;
    it('creating a new contact with only needed params should return 200', async () => {
        const res = await axios.post(base + '/api/contacts', {
            "firstname": "first",
            "lastname": "last"
        }, axios_config);
        added_contact = res.data;
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json")
    });
    it('valid update should return 200', async () => {
        let contact_copy = added_contact
        contact_copy.firstname = "second"
        const res = await axios.put(base + '/api/contacts/' + added_contact.id, contact_copy, axios_config);
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json")
        expect(res.data).toEqual(contact_copy);
    });
});
// ---------------
describe('Update contact id after adding(should fail)', () => {
    let added_contact;
    it('creating a new donor with only needed params should return 200', async () => {
        const res = await axios.post(base + '/api/contacts', {
            "firstname": "first",
            "lastname": "last"
        }, axios_config);
        added_contact = res.data;
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json")
    });
    it('invalid update should return 406', async () => {
        added_contact.id++;
        const res = await axios.put(base + '/api/contacts/' + (added_contact.id - 1), added_contact, axios_config);
        expect(res.status).toEqual(406);
        expect(res.headers['content-type']).toContain("application/json")
    });
});
// ---------------
describe('Update contact group after adding (should work)', () => {
    let added_org;
    let added_team;
    let added_contact;
    it('creating a new org with just a name should return 200', async () => {
        const res = await axios.post(base + '/api/organizations', {
            "name": "test123"
        }, axios_config);
        delete res.data.contact;
        delete res.data.teams;
        added_org = res.data
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json")
    });
    it('creating a new team with a parent org should return 200', async () => {
        const res = await axios.post(base + '/api/teams', {
            "name": "test_team",
            "parentGroup": added_org.id
        }, axios_config);
        delete res.data.contact;
        added_team = res.data;
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json")
    });
    it('creating a new contact with a valid org should return 200', async () => {
        const res = await axios.post(base + '/api/contacts', {
            "firstname": "first",
            "lastname": "last",
            "groups": added_org.id
        }, axios_config);
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json");
        added_contact = res.data;
        delete res.data.groups[0].teams;
        expect(res.data).toEqual({
            "id": res.data.id,
            "firstname": "first",
            "middlename": null,
            "lastname": "last",
            "phone": null,
            "email": null,
            "address": {
                "address1": null,
                "address2": null,
                "postalcode": null,
                "city": null,
                "country": null
            },
            "groups": [added_org],
            "responseType": "GROUPCONTACT"
        });
    });
    it('valid group update to single team should return 200', async () => {
        const res = await axios.put(base + '/api/contacts/' + added_contact.id, {
            "id": added_contact.id,
            "firstname": "first",
            "lastname": "last",
            "groups": added_team.id
        }, axios_config);
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json");
        expect(res.data).toEqual({
            "id": res.data.id,
            "firstname": "first",
            "middlename": null,
            "lastname": "last",
            "phone": null,
            "email": null,
            "address": {
                "address1": null,
                "address2": null,
                "postalcode": null,
                "city": null,
                "country": null
            },
            "groups": [added_team],
            "responseType": "GROUPCONTACT"
        });
    });
    it('valid group update to org and team should return 200', async () => {
        const res = await axios.put(base + '/api/contacts/' + added_contact.id, {
            "id": added_contact.id,
            "firstname": "first",
            "lastname": "last",
            "groups": [added_org.id, added_team.id]
        }, axios_config);
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json");
        delete res.data.groups[0].teams;
        expect(res.data).toEqual({
            "id": res.data.id,
            "firstname": "first",
            "middlename": null,
            "lastname": "last",
            "phone": null,
            "email": null,
            "address": {
                "address1": null,
                "address2": null,
                "postalcode": null,
                "city": null,
                "country": null
            },
            "groups": [added_org, added_team],
            "responseType": "GROUPCONTACT"
        });
    });
    it('valid group update to none should return 200', async () => {
        const res = await axios.put(base + '/api/contacts/' + added_contact.id, {
            "id": added_contact.id,
            "firstname": "first",
            "lastname": "last",
            "groups": null
        }, axios_config);
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json");
        expect(res.data).toEqual({
            "id": res.data.id,
            "firstname": "first",
            "middlename": null,
            "lastname": "last",
            "phone": null,
            "email": null,
            "address": {
                "address1": null,
                "address2": null,
                "postalcode": null,
                "city": null,
                "country": null
            },
            "groups": [],
            "responseType": "GROUPCONTACT"
        });
    });
});
// ---------------
describe('Update contact group invalid after adding (should fail)', () => {
    let added_org;
    let added_contact;
    it('creating a new org with just a name should return 200', async () => {
        const res = await axios.post(base + '/api/organizations', {
            "name": "test123"
        }, axios_config);
        delete res.data.contact;
        delete res.data.teams;
        added_org = res.data
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json")
    });
    it('creating a new contact with a valid org should return 200', async () => {
        const res = await axios.post(base + '/api/contacts', {
            "firstname": "first",
            "lastname": "last",
            "groups": added_org.id
        }, axios_config);
        expect(res.status).toEqual(200);
        expect(res.headers['content-type']).toContain("application/json");
        added_contact = res.data
        delete res.data.groups[0].teams;
        expect(res.data).toEqual({
            "id": res.data.id,
            "firstname": "first",
            "middlename": null,
            "lastname": "last",
            "phone": null,
            "email": null,
            "address": {
                "address1": null,
                "address2": null,
                "postalcode": null,
                "city": null,
                "country": null
            },
            "groups": [added_org],
            "responseType": "GROUPCONTACT"
        });
    });
    it('invalid group update to single team should return 404', async () => {
        const res = await axios.put(base + '/api/contacts/' + added_contact.id, {
            "id": added_contact.id,
            "firstname": "first",
            "lastname": "last",
            "groups": 999999999999999
        }, axios_config);
        expect(res.status).toEqual(404);
        expect(res.headers['content-type']).toContain("application/json");
    });
});